;;; e-catdoc.el --- A wrapper around catdoc's Microsoft Word to Text converter
;; Time-stamp: <2002-03-05 22:08:06 bingalls>
;;$Id$
;; Copyright  1998 by Free Software Foundation, Inc.

;; Author: Steve Kemp <skx@tardis.ed.ac.uk>
;; Modified by Bruce Ingalls <bingalls@sf.net>. Original code in word.el.gz

;; Keywords: local, tools, MS Word, conversion, text
;;
;; This is a simple wrapper to the Word to Text conversion program catdoc.
;; catdoc is freely available from :
;;	<URL:http://www.fe.msk.ru/~vitus/catdoc/>
;;
;;  word.el is currently available from
;;	<url:http://GNUsoftware.com/Emacs/Lisp/>
;;	<url:http://www.tardis.ed.ac.uk/~skx/win/>
;;  [Much of the create process code was borrowed / inspired by gnustart.el.]
;;
;;  Steve Kemp
;;  Tue May 11 09:59:56 1999  -Bruce: See timestamp above
;;;;;;

;;Code added by Bruce
(require 'which)
(require 'e-functions)	;which-first()
;(require 'easymenu)

;;Modified from hard-coded "catdoc" by Bruce
(defvar catdoc-prog (which-first "catdoc")
  "The name of the program that will perform the conversion.")

;;  Any optional arguments you'd like to pass to catdoc on a regular basis
;; should be defined here.

(defvar catdoc-args ""
  "Any additional options to pass to catdoc.")

(defvar catdoc-add-menu-item t
  "Controls whether or not to add a new entry on the file menu, that will allow
 word files to be opened from the GUI.")

(defvar catdoc-process nil
  "The catdoc process that we are going to spawn.")

(defvar catdoc-filename nil
  "The input filename to the catdoc process, this is used later to allow rename
 the output buffer appropriately.")

(defun catdoc-process-filter (proc string)
  "Process the results from the catdoc process."
  (set-buffer (get-buffer-create "*catdoc*"))
  (insert string))

(defun catdoc-sentinel (proc msg)
  "When the catdoc process has finished, switch to its output buffer,
 and rename it appropriately."
  (cond ((eq (process-status proc) 'exit)
	 (switch-to-buffer "*catdoc*")
	 (rename-buffer (concat catdoc-filename ".txt"))
	 )))

(defun open-word-document (file-name )
  "Find, decode and display a specific Microsoft Word document."
  (interactive "fOpen Word Document: \n")
  (setq file-name (expand-file-name file-name))
  (setq catdoc-filename file-name)
  (setq catdoc-process
	(start-process "catdoc" nil catdoc-prog catdoc-args file-name))
;  (set-process-filter catdoc-process 'catdoc-process-filter)
  (set-process-sentinel catdoc-process 'catdoc-sentinel)
  (process-kill-without-query catdoc-process)
  )

;;Bruce: modified original menu code to also work on XEmacs
(defconst ms-word-menu
  '("Import"
    ["MS-Word"	open-word-document t])
  "Menu for reading in Microsoft Word documents.")

(autoload 'add-submenu "menubar" "menubar" t)	;shut up compiler

(if (and (string-match "XEmacs" (emacs-version))
	 (and catdoc-add-menu-item (functionp 'add-submenu)))
    (progn
      (add-submenu '("File") ms-word-menu "Import MS Word")
      (defvar menu-bar-file-menu))	;shut up compiler
  (if (and catdoc-add-menu-item (string-match "GNU" (emacs-version)))
      (progn
;	(require 'menu-bar)
	(define-key menu-bar-file-menu [catdoc-open]
	  '("Import MS Word" . open-word-document)))))

;;Bruce: Could not get this portable solution working:
;	     (easy-menu-add-item nil '("files")
;				 (easy-menu-create-menu (car ms-word-menu)
;							(car ms-word-menu)))


;;  Allow others to use this package.
(provide 'e-catdoc)
