;;; e-common.el --- Enhancements for both GNU and X Emacs on all platforms
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <02/05/22 17:28:42 bingalls>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;	TABLE OF CONTENTS
;; 	Legal
;; 	Commentary
;; 	Code
;;	Applications
;;		Calendar
;;		Dictionary
;;		Power Macros
;;		Spell Checker
;;		Tiny Tools
;;	Beginner's Settings
;;	Devices
;;		Alternate Character Set
;;		Mouse Set Up
;;		Printer Set Up
;;	Display & Menu
;;		Cycle Buffers
;;		Horizontal Scroll Bar
;;	File Handling
;;		Backups in ~/.backups Directory
;;		Compressed Files
;;			Bzip Compressed Files
;;	Modes
;;		Completions
;;		Info Mode Font Locking/Syntax Coloring
;;		Info Path Appending
;;		Shell
;;		Word Completion / Dabbrev
;;	Misc
;;		Tiny Replace
;;		Force Single Space After Each Period
;;	Security
;;	Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;;; Commentary:
;;Emacs editor startup file.  Power User routines for every OS, for both Gnu &
;;X Emacs

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://www.sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;;; Keywords:		advanced power user enhanced miscellaneous applications

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-common nil "Settings from e-common.el file." :group 'emacro)

(require 'which)
(cache-locate-library
 use-cache 'cache-dired "dired"
 "For printing support, get dired.el from full XEmacs sumo tarball")
(when cache-dired (require 'dired))

(require 'cus-edit)

;;__________________________________________________________________________
;;;;	Applications
;;__________________________________________________________________________
;;;;;;		Calendar
(defcustom mark-diary-entries-in-calendar t
  "This takes effect when you invoke 'M-x calendar'."
  :group 'e-common
  :type 'boolean)
(defcustom mark-holidays-in-calendar t "Display holidays."
  :group 'e-common
  :type 'boolean)
(defcustom diary-file "~/.diary" "Display holidays."
  :group 'e-common
  :type 'string)
(defconst diary-display-hook 'fancy-diary-display)
(defconst calendar-setup 'one-frame)
(autoload 'calendar "calendar" "Display 3-month calendar." t)

;;(setq calendar-latitude 40.7)		;New York City
;;(setq calendar-longitude -74.0)	;New York City, EDT

;;(require 'appt)
;;(appt-initialize)

;;See i18n for European calendar style

;;__________________________________________________________________________
;;;;;;		Dictionary
(cache-locate-library
 use-cache 'cache-dictionary "dictionary"
 "For dictionary support, get dictionary.el from <url:http://www.in-berlin.de/User/myrkr/dictionary.html> or <url:http://www.dict.org>")

(when cache-dictionary
  (defcustom dictionary-server "localhost" "URI of online dictionary."
    :group 'e-common
    :type 'string)
  (global-set-key [(alt m)] 'dictionary-match-words)
  )

;;__________________________________________________________________________
;;;;;;		Power Macros
;;Improved library to record keystrokes, et. al.
;;To configure, 'M-x customize-group RET power-macros'
;;<url:http://www.imada.sdu.dk/~blackie/emacs/>
(autoload 'power-macros-mode "power-macros-mode" "power macros" t)

;;Jaari Alto's macro tool assigns F9 & C-F9 :
(autoload 'tinymacro-end-kbd-macro-and-assign  "tinymacro" t t)

;;__________________________________________________________________________
;;;;;;		Spell Checker
;;Find ispell in PATH. Note that version 3.x is more recent than v4!
(cond ((which "ispell")
       (progn
	 (defcustom ispell-el "ispell" "Program name of spell checker."
	   :group 'e-common
	   :type 'string)))
      ((which "ispell4")
       (defcustom ispell-el "ispell4" "Program name of spell checker."
	 :group 'e-common
	 :type 'string))
      (t (message "For spell-check support, get ispell. See emacs.html for sources")))

;;__________________________________________________________________________
;;;;;;		Tiny Tools
;;Enhancements to programming, search/replace, email and much more
(cache-locate-library
 use-cache 'cache-tiny-setup "tiny-setup"
 "For overall enhancement, get tiny-tools from <url: http://tiny-tools.sourceforge.net>")
(when cache-tiny-setup (require 'tiny-setup))

;;__________________________________________________________________________
;;;;	Beginner's Settings
;;These variables may make emacs easier to pick up.
(put 'set-goal-column 'disabled nil)
(put 'narrow-to-page 'disabled nil)
(setq enable-recursive-minibuffers t)
(put 'eval-expression 'disabled nil)
(defcustom query-replace-highlight t "Highlight during query."
  :group 'e-common
  :type 'boolean)
(defcustom search-highlight t "Hilight incremental search."
  :group 'e-common
  :type 'boolean)
;;enable converting selected text to lower case by default
(put 'downcase-region 'disabled nil)
;;enable converting selected text to upper case by default
(put 'upcase-region 'disabled nil)

;;__________________________________________________________________________
;;;;	Devices

;;__________________________________________________________________________
;;;;;;		Mouse Set Up
;;(setq mouse-yank-at-point 't)	;Paste at text point NOT at mouse cursor

;; Scroll Bar gets dragged by mouse butn 1
(global-set-key [vertical-scroll-bar down-mouse-1] 'scroll-bar-drag)

;; Rebind mouse-2 events to mouse-1 in various places:
;; Completion list
(add-hook 'completion-list-mode-hook
          '(lambda() (define-key completion-list-mode-map [down-mouse-1]
                       'mouse-choose-completion)))
;; TexInfo
(add-hook 'Info-mode-hook
          '(lambda() (define-key Info-mode-map [down-mouse-1]
                       'Info-mouse-follow-nearest-node)))
;; Buffer Menu
(add-hook 'buffer-menu-mode-hook
          '(lambda() (define-key Buffer-menu-mode-map [down-mouse-1]
                       'Buffer-menu-mouse-select)))

;;(global-unset-key [mouse-2])	;disable intellimouse wheel

;;__________________________________________________________________________
;;;;;;		Printer Set Up
(cache-locate-library
 use-cache 'cache-lpr "lpr"
 "For printing support, get lpr.el from full XEmacs sumo tarball")
(when cache-lpr
  (autoload 'lpr-buffer "lpr" "lpr print." t)
  (autoload 'lpr-region "lpr" "lpr print." t))

;;XEmacs can't handle this:
;;(cache-locate-library
;; use-cache 'cache-tinylpr "tinylpr"
;; "For printers menu, get tinylpr from <url: http://tiny-tools.sourceforge.net/ >")
;;(when cache-tinylpr (require 'tinylpr))

;;__________________________________________________________________________
;;;;	File Handling
;;(setq auto-save-directory (expand-file-name "~/.autosave/"))
;;(require 'auto-save)

;;__________________________________________________________________________
;;;;;;		Backups in ~/.backups Directory
(defvar cache-ebackup)		;shut up compiler
(cache-locate-library
 use-cache 'cache-ebackup "ebackup"
 "To put all your *~ backups into ~/.backup, get ebackup.el from <url:http://relativity.yi.org>")
;;Need to find out why this code does not work with gnuserv on w32
(when (and cache-ebackup (not (which "gnuclientw")))
  (require 'ebackup)
  (defcustom make-backup-files t "Store backups in '~/.backup'."
    :group 'e-common
    :type 'boolean))

;;__________________________________________________________________________
;;;;;;		Compressed Files
;;XEmacs says that jka-compr is obsolete. However it is portable with Gnu
(defvar cache-jka-compr)		;shut up compiler
(cache-locate-library
 use-cache 'cache-jka-compr "jka-compr"
 "For compressed file support, get jka-compr.el from the full tarball distribution where you got emacs")

(condition-case err		;Handle any exceptions
    (autoload 'toggle-auto-compression "jka-compr" "auto decompress files" t)
  (error
    (autoload 'auto-compression-mode "jka-compr" "auto decompress files" t)))

(when cache-jka-compr
  ;;This doesn't autoload nicely, but it sometimes shuts up bytecompiler
  (require 'jka-compr)                  ;read compressed files
  (condition-case err		;Handle any exceptions
      (toggle-auto-compression t)
    (error
     (auto-compression-mode t))))          ;automatically decompress files
(autoload 'archive-mode "arc-mode" "Major mode for editing archives." t)
(autoload 'tar-mode "tar-mode" "Major mode for editing Tape ARchives." t)

(when (which "unzip")
  (defcustom archive-zip-use-pkzip nil "When nil, use infozip not pkzip."
    :group 'e-common
    :type 'boolean))
(defcustom jka-compr-use-shell nil
  "Discard stderr compression output. Always nil for w32."
  :group 'e-common
  :type 'boolean)

;;__________________________________________________________________________
;;;;;;;;		Bzip2 Compressed Files
(defvar Info-suffix-list)
(defun setup-bzip2 () "Read bzip2 compressed info files."
  (progn
    (nconc Info-suffix-list '((".info.bz2" . "bzip2 -dc %s")))
    (nconc Info-suffix-list '((".bz2" . "bzip2 -dc %s")))))
(add-hook 'Info-mode-hook 'setup-bzip2)

;;__________________________________________________________________________
;;;;	Display & Menu
(defcustom icon-title-format "Emacs - %b"
  "Filename alone fits into icon. 'Emacs - %f' shows full path."
  :group 'e-common
  :type 'string)

(defconst system-name (system-name))
(defcustom frame-title-format '(" " system-name ":" default-directory " %12b")
  "Host:/path/[filename | nonfile-mode] in title bar. Mode=*shell*,*grep*,etc."
  :group 'e-common
  :type 'string)

(column-number-mode t)			;column number in modeline (status)
(line-number-mode t)			;line number in modeline (status bar)
(autoload 'display-time "time" "clock in status bar" t) ;shut up compiler

(defvar cache-time)			;shut up compiler
(cache-locate-library
 use-cache 'cache-time "time"
 "For clock display support, get time.el from the full tarball distribution where you got emacs")

(when cache-time
  (require 'time)
  (display-time))			;clock (sometimes in status bar)

;;See i18n for European/Military style time

(defvar use-left)			;shut up compiler
(defvar use-top)			;shut up compiler
(defvar use-height)			;shut up compiler
;;Position emacs, unless this is an ascii terminal
(when window-system
  (set-frame-position (selected-frame) use-left use-top)

  ;;If this does not set emacs height, then try it twice:
  (set-frame-height (selected-frame) use-height)
  (set-frame-width (selected-frame) 80)

  (add-to-list 'default-frame-alist '(width . 80))
  )

;;Prevent passwords in M-x shell from being seen.
(add-hook 'comint-output-filter-functions 'comint-watch-for-password-prompt)

(defcustom default-mode-line-format
  '("-"
    (line-number-mode "L%l--")
    (column-number-mode "C%c--")
    mode-line-mule-info mode-line-modified
    mode-line-frame-identification mode-line-buffer-identification
    "   " global-mode-string "   %[(" mode-name mode-line-process
    minor-mode-alist "%n" ")%]--"
    (which-func-mode ("" which-func-format "--"))
    "-%-")
  "Default display of status bar.
   May be overridden by jde-mode-line-format and other modes."
  :group 'e-common
  :type 'string)

;;__________________________________________________________________________
;;;;;;		Cycle Buffers
;;ToDo: C-F6/C-S-F6 is the w32 keybinding way
(defvar cache-iswitchb)			;shut up compiler
(cache-locate-library
 use-cache 'cache-iswitchb "iswitchb"
 "To cycle buffers with Control (Shift) TAB, get iswitchb.el from <url:http://www.cns.ed.ac.uk/people/stephen/emacs/>")
(autoload 'pc-bufsw::bind-keys "pc-bufsw" "pc-bufsw" t)

(when cache-iswitchb
  (autoload 'iswitchb-buffer "iswitchb" "iswitchb" t)
  (global-set-key [(control tab)] 'iswitchb-buffer)
  )

;;__________________________________________________________________________
;;;;;;		Horizontal Scroll Bar
;;These routines prevent line wrap-around (marked by continuation glyphs/chars)
;;(load-library "scroll-mode")
;;(setq-default auto-show-mode t)
;;(toggle-horizontal-scroll-bar t)	;;Might not yet be available.

;;__________________________________________________________________________
;;;;	Modes

;;__________________________________________________________________________
;;;;;;		Completions
(require 'icomplete)
(icomplete-mode)

;;Klaus Berndl	<klaus.berndl@sdm.de>
;;We want to complete and expand as much as possible in all buffers.
;;Order of try-functions is significant! Includes the whole dabbrev-stuff.
(defvar cache-hippie-exp)		;shut up compiler
(cache-locate-library
 use-cache 'cache-hippie-exp "hippie-exp"
 "For printing support, get hippie-exp.el from full XEmacs sumo tarball")
(autoload 'hippie-expand "hippie-exp" "hippie-exp" t)

(when cache-hippie-exp
  (require 'hippie-exp)
  (setq hippie-expand-try-functions-list
        '(
          ;;try-complete-tempo-tag	;not active for me
          try-expand-dabbrev
          try-expand-dabbrev-all-buffers
          try-expand-dabbrev-from-kill
          try-complete-file-name-partially
          try-complete-file-name
          ;;        try-expand-list
          try-complete-lisp-symbol-partially
          try-complete-lisp-symbol
          try-expand-whole-kill))
  ;; the expand-function. Called with a positive prefix <P> it jumpes direct
  ;; to the <P>-th try-function.
  (defun my-hippie-expand (arg)
    (interactive "P")
;;hippie-expand doesn't have customization-feature (like dabbrev-expand) to
;;search case-sensitive for completions. So set 'case-fold-search' temp to nil
;;REMOVE THIS LET STATEMENT!
;;    (let ((old-case-fold-search case-fold-search))
      (hippie-expand arg)
;;      )
    )
  ;;C-/ is used, this is S-C-/
  (global-set-key [(control \?)] (quote my-hippie-expand)))

;;__________________________________________________________________________
;;;;;;		Info Mode Font Locking/Syntax Coloring
(defvar info-font-lock-keywords
  (list
   '("^\\* [^:]+:+" . font-lock-function-name-face)
   '("\\*[Nn]ote\\b[^:]+:+" . font-lock-reference-face)
   '("  \\(Next\\|Prev\\|Up\\):" . font-lock-reference-face))
  "Additional expressions to highlight in Info mode.")

(add-hook 'Info-mode-hook
          (lambda ()
            (make-local-variable 'font-lock-defaults)
            (defconst font-lock-defaults '(info-font-lock-keywords nil t))))

;;__________________________________________________________________________
;;;;;;		Info Path Appending
;;(setq Info-directory-list
;;      (append '("/usr/local/lib/info")
;;	      (or (and (boundp 'Info-directory-list)
;;		       Info-directory-list)
;;		  (and (boundp 'Info-default-directory-list)
;;		       Info-default-directory-list)
;;		  (list Info-directory))))

;;Alternate:
;;(setq Info-default-directory-list
;;      (cons "c:/cvs/doc/cvsclient.texi" Info-default-directory-list))

;;__________________________________________________________________________
;;;;;;		Shell
;;Use .emacs_shellname for the shells rc file
;;For XEmacs, this requires ansi-color v3.3 or higher.
(autoload 'ansi-color-for-comint-mode-on "ansi-color" "ansi-color" t)
(defvar cache-ansi-color)		;shut up compiler
(cache-locate-library
 use-cache 'cache-ansi-color "ansi-color"
 "For color in your M-x shell, get ansi-color from <url:http://www.gnu.org/>")

(when cache-ansi-color
  (autoload 'ansi-color-for-comint-mode-on "ansi-color" nil t)
  (add-hook 'shell-mode-hook 'ansi-color-for-comint-mode-on))

;;__________________________________________________________________________
;;;;;;		Word Completion / Dabbrev
;;causes M-/ to fill in the rest of a word
;;optional package for XEmacs in sumo tarball
(autoload 'dabbrev-expand "dabbrev" "M-/ fills in the rest of a word" t)
(autoload 'dabbrev-completion "dabbrev" "M-/ fills in the rest of a word" t)

(defcustom dabbrev-always-check-other-buffers t
  "Grab word completion vocabulary from all active sessions."
  :group 'e-common
  :type 'boolean)

(defcustom dabbrev-abbrev-char-regexp "\\sw\\|\\s_"
  "Regular expression to capture word completions."
  :group 'e-common
  :type 'string)

(autoload 'quietly-read-abbrev-file "abbrev" "word completion" t)
(autoload 'abbrev-mode "abbrev" "word completion" t)

;;From emacs faq. See it, to learn more.
(condition-case () (quietly-read-abbrev-file) (file-error nil))
(add-hook 'text-mode-hook (function (lambda () (setq abbrev-mode t))))

;;abbrev
(autoload 'read-abbrev-file "abbrev" "word completion" t)
;;(defconst abbrev-file-name "~/.abbrev_defs")	;default from write-abbrev-file
(if (file-exists-p abbrev-file-name)
    (read-abbrev-file))

;;__________________________________________________________________________
;;;;	Misc
;;Prevent Emacs (not a XEmacs problem) from appending, when moving down at EOF
(setq next-line-add-newlines nil)

(cache-locate-library
 use-cache 'cache-speedbar "speedbar"
 "For fast file menu support, get speedbar from <URL:http://www.ultranet.com/~zappo/speedbar.shtml>")

(autoload 'speedbar-frame-mode "speedbar" "Popup a speedbar frame" t)
(autoload 'speedbar-get-focus "speedbar" "Jump to speedbar frame" t)

(fset 'yes-or-no-p 'y-or-n-p)		;replace y-e-s by y
(defalias 'yes-or-no-p 'y-or-n-p)
;;(setq tex-dvi-view-command "xdvi")
(defcustom font-menu-ignore-scaled-fonts nil "Mostly for XEmacs?"
  :group 'e-common
  :type 'boolean)

;;__________________________________________________________________________
;;;;;;		Force Single Space After Each Period
(defconst sentence-end "[.?!][]\"')}]*\\($\\|[ \t]\\)[ \t\n]*")
(defcustom sentence-end-double-space nil
  "When nil, force single space after each period."
  :group 'e-common
  :type 'boolean)

;;__________________________________________________________________________
;;;;;;	Security
;; (defcustom package-get-require-base-signed-base-updates 'nil
;;   "When nil, stop pgp package checking."
;;   :group 'e-common
;;   :type 'boolean)

;;__________________________________________________________________________
;;;;	Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-common)

;;; e-common.el ends here
