;;;; e-config.el --- Define user variables.
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <2002-06-02 08:50:26 bingalls>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;    TABLE OF CONTENTS
;;      Legal
;;      Commentary
;;      Code
;;	Defining First Time Cache Variables
;;	Notices At Top Of New Generated Files
;;	System Probing
;;	Variables
;;		Set Your Startup Size And Position
;;		Color & Size
;;		Internetworking
;;		Printer
;;		Programmer
;;		SQL Mode
;;	Load Affiliated Files
;;	Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;; Commentary:
;;Set your personalized global variables here
;;Put any extensive code in a separate file, to make upgrades easier.
;;
;;Typical values are "t" or "'t" for True (no "" double quotes) or
;;  "'nil" or "nil" for false
;;Example:
;;  (defconst use-modem t)		;true: I connect to web via modem
;;  (defconst use-modem nil)		;false: I connect via ethernet

;;BE SURE TO USE A VALUE. DO NOT COMMENT OUT THESE CHOICES, as you will get
;;bytecompile errors, requiring you to hack the code throughout EMacro!
;;
;;Limit your editing of this file, as it is due to change in the future.
;;Instead, put your customizations in ~emacs/e-preload.el, e-postload.el,
;;e-custom.el (emacs), or e-xcustom.el (xemacs)

;;ToDo? Extract server names from Mozilla or Netscape's prefs.js

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://www.sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;; Keywords:		user configure configuration setup

;;__________________________________________________________________________
;;;; Code:
;;Some of this shuts up warnings from the elisp byte-compiler:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-config nil "Settings from e-config.el file." :group 'emacro)

(setq max-specpdl-size 8000)
;;(setq max-specpdl-size 99999)
(defcustom max-specpdl-size 8000
  "XEmacs 21.4 with debug requires more than default.
8000 is Normally big enough."
  :group 'e-config
  :type 'integer)

(setq max-lisp-eval-depth 22000)
(defcustom max-lisp-eval-depth 22000
  "This large size prevents w32 XEmacs lockup and JDE (Semantic?) woes."
  :group 'e-config
  :type 'integer)

;;Some user .emacs code works better, before loading the bulk of EMacro.
;;Simulate a require(), in case users wipe out the file.
(when (not (featurep 'e-preload))
  (load-library "e-preload")
  (provide 'e-preload))

(require 'which)                        ;works in my MacOS port.
(autoload 'cache-locate-library "functions" "optimized locate-library" t)

(defconst use-browser 'nil)		;initialize for inet.el

;;__________________________________________________________________________
;;;;		Notices At Top Of New Generated Files
(emacro-set-file (concat emacro-top-dir "preferences/e-cache.el"))
(emacro-set-file (concat emacro-top-dir "preferences/e-xcache.el"))

;;__________________________________________________________________________
;;;;		System Probing
(autoload 'which "which" "find file in path" t)	;shut up bytecompiler
;;load any programs previously found in the path

;;these function definitions may be required
(require 'e-functions)

;;These files must be loaded before most of EMacro.
(defconst pref-file (concat emacro-top-dir "preferences/e-prefs.el"))
(if (file-exists-p pref-file)
  (load-library "e-prefs")
  (emacro-set-file pref-file))

;;__________________________________________________________________________
;;;;	Variables
;;Note that variables are set, at the same time they write to preferences.el,
;;so that EMacro works from the first startup.

;;__________________________________________________________________________
;;;;;;		Set Your Startup Size And Position
;;Make sure that user can see the prompt at bottom, before continuing.
(defvar use-height)			;shut up compiler
(defvar use-left)			;shut up compiler
(defvar use-top)			;shut up compiler
(autoload 'frame-char-height "css" "char height" t)

(when (not (boundp 'use-height))
  (if (eq window-system 'x)
;;use-auto-height is a temp variable for automatic system determined height
      (progn
	(defconst use-auto-height
	  (/ (- (x-display-pixel-height) 50) (frame-char-height)))
	(setq use-height use-auto-height))
;;Else not X Window, and can't autosize. Init height to 24, so prompt is seen:
    (progn
      (set-frame-height (selected-frame) 24)
      (append-to-file
       (format "(setq use-height %s)\n"
	       (setq use-height
		     (string-to-number
		      (read-string
		       "What height should Emacs start at? " "40"))))
       'nil pref-file))))

(when (or (not (boundp 'use-top)) (not (boundp 'use-left)))
  (setq use-top 0)
  (setq use-left 60)
  ;;60 pixels allows desktop taskbar on left, & speedbar also

  ;;Classic MacOS: use-top should be at least 38 to prevent obscuring title bar
  (cond ((string-match "Macintosh" (emacs-version))
         (setq use-top (+ use-top 40)))
	 ((string-match "macos" (emacs-version))
	  (setq use-top 0))

        ;;Else do shell-command-to-string() requires emacs > 20.1?

        ;;This probe slows EMacro by ~1 second, but happens only first time.
        ;;enlightenment & fvwm95: use-top > 8 prevents obscuring title bar
        (t
         (defconst use-ps "")
	 (when (which "ps")
	   (setq use-ps (shell-command-to-string "ps -u $USER")))
	 (setq use-top 0)
	 (setq use-left 60)
	 (when
	     (string-match "englightenment\\|fvwm95\\|afterstep" use-ps)
           (setq use-top (+ use-top 8))
           (when (boundp 'use-auto-height)
             (setq use-height (- use-auto-height use-top))))

         ;;Sawmill is obsolete, and this section may disappear
	 (when (string-match "sawmill" use-ps)
           (setq use-left (+ use-left 40))
           (setq use-top (+ use-top 26))

           (when (boundp 'use-auto-height)
             (setq use-height (- use-auto-height use-top))))

         ;;New versions of Sawfish correctly offset the top of emacs
	 (when (string-match "sawfish" use-ps)
           (defconst sawfish-ver
             (shell-command-to-string "sawfish --version"))
           (cond
            ((string-lessp (caddr (split-string sawfish-ver)) "0.32")
             (setq use-left (+ use-left 40))
             (setq use-top (+ use-top 26))

             (when (boundp 'use-auto-height)
               (setq use-height (- use-auto-height use-top))))
            (t
             (when (boundp 'use-auto-height)
               (setq use-height (- use-auto-height 4))))))


	 (append-to-file (format "(setq use-top %s)\n" use-top) 'nil pref-file)
	 (append-to-file (format "(setq use-left %s)\n" use-left) 'nil
			 pref-file))))

(when (boundp 'use-auto-height)
  (append-to-file (format "(setq use-height %s)\n"  use-auto-height) 'nil
		  pref-file))

;;__________________________________________________________________________
;;;;		Color & Size
;;This code will not tinyload very well :(
(cache-locate-library
 use-cache 'cache-color-theme "color-theme"
 "To start EMacro faster, get tiny tools from <url: http://www.emacswiki.org/cgi-bin/wiki.pl?ColorTheme>")
(defvar default-frame-plist)		;shut up compiler

(if cache-color-theme
    (when (not (featurep 'color-theme))
      (progn
	(require 'color-theme)		;setup color-theme for first time
	(autoload 'color-theme-standard "color-theme" "color-theme" t)
	(color-theme-standard)		;afterwards, use preferences.el
	(append-to-file
	 "(require 'color-theme)\n(color-theme-standard)\n" 'nil pref-file)))
;;else set up some default colors
  (if (string-match "XEmacs" (emacs-version)) ;Won't compile for xemacs < v21
      (progn
	(setq frame-initial-frame-plist `(height ,use-height))
;;THESE DEFAULTS WORK NICELY ON MOST MACHINES
;;	(add-to-list 'default-frame-plist '(foreground-color . "Black"))
;;	(add-to-list 'default-frame-plist '(background-color . "Grey90"))
	)
;;Else Emacs
    (progn
      (setq default-frame-plist `(height . ,use-height))
;;THESE DEFAULTS WORK NICELY ON MOST MACHINES
;;      (add-to-list 'default-frame-alist '(foreground-color . "Black"))
;;      (add-to-list 'default-frame-alist '(background-color . "Grey90"))
      )))

;;Color now set. Next, set font size & family, depending on OS
;;(when (string-match "GNU" (emacs-version)
;;       (if (string-match "windows" (symbol-name system-type))
;;	   (set-default-font "-courier-new-*-*-*-10-*-*-*-*-*-*-*")
;; ;;Otherwise, not ms windows: use a larger, unix font
;;	 (set-default-font "-adobe-courier-*-*-*-12-*-*-*-*-*-*-*"))))

;;__________________________________________________________________________
;;;;;;		Internetworking
(defvar use-net)			;shut up compiler
(when (not (boundp 'use-net))
  (if (y-or-n-p
       "Do you want Emacs to access the internet? (email, news) ")
         ;;Append the results to preferences.el
      (progn
	(setq use-net 't)
	(append-to-file "(setq use-net 't)\n" 'nil pref-file))
;;else use-net is false
    (progn
      (setq use-net 'nil)
      (append-to-file "(setq use-net 'nil)\n" 'nil pref-file))))

;;This optimizes your connection. Ex: tiny-url will cache urls, until
;;you redial your ISP, instead of forcing an immediate connection.
(defvar use-modem)			;shut up compiler
(when (and (not (boundp 'use-modem)) use-net)
  (if (y-or-n-p
       "Do you have fast access to the internet? (no if dialup modem) ")
      ;;Append the results to preferences.el
      (progn
	(setq use-modem 'nil)
	(append-to-file "(setq use-modem 'nil)\n" 'nil pref-file))
;;else
    (progn
      (setq use-modem 't)
      (append-to-file "(setq use-modem 't)\n" 'nil pref-file))))

;;use-login-name is needed, even if we don't connect to net
;;could be used for databases
(defvar use-login-name)			;shut up compiler
(when (not (boundp 'use-login-name))
  (cond
   ((getenv "USER");;First try unix environment variables
    (append-to-file
     (format "(setq use-login-name \"%s\")\n"
             (setq use-login-name (getenv "USER")))
     'nil pref-file))

   ((getenv "LOGNAME")
    (append-to-file
     (format "(setq use-login-name \"%s\")\n"
             (setq use-login-name (getenv "LOGNAME")))
     'nil pref-file))

   ((getenv "USERNAME")                 ;Next try w32 environment variables:
    (append-to-file
     (format "(setq use-login-name \"%s\")\n"
             (setq use-login-name (getenv "USERNAME")))
     'nil pref-file))

   (t			      ;Could be mac, which doesn't set these variables:
    (append-to-file
     (format "(setq use-login-name \"%s\")\n"
             (setq use-login-name
                   (read-string
		    "What's your login name? [for net, sql] " "userID")))
     'nil pref-file))))

(defvar user-full-name)			;shut up compiler
(when (and (not (boundp 'user-full-name)) use-net)
  (append-to-file
   (format "(setq user-full-name  \"%s\")\n"
           (setq user-full-name
                 (read-string "What's your full name? " "Joe Novice")))
   'nil pref-file))

;;Save user's return email address in preferences
(defvar gnus-nntp-server)               ;shut up compiler
(defvar use-smtp)			;shut up compiler
(defvar use-pop3)			;shut up compiler
(when use-net
  (when (or (or (or
                 (not (boundp 'user-mail-address))
                 (not (boundp 'use-smtp)))
                (not (boundp 'use-pop3)))
            (not (boundp 'gnus-nntp-server)))
    (when (getenv "MAILHOST")
      (defconst use-domain
            (format "%s.%s" 
                    (car(cdr (split-string (getenv "MAILHOST") "[.]")))
                    (car(cdr(cdr (split-string (getenv "MAILHOST") "[.]")))))))

    (when (not (boundp 'use-domain))
      (defconst use-domain
            (read-string "What's your network domain? " "localhost.com")))))

(when (and (not (boundp 'user-mail-address)) use-net)
  (append-to-file
   (format
    "(setq user-mail-address \"%s\")\n"
    (setq user-mail-address
          (read-string "What's your email address? "
                       (format "%s@%s" use-login-name use-domain))))
   'nil pref-file))

;;address of your smtp email server to SEND mail:
(when (and (not (boundp 'use-smtp)) use-net)
  (append-to-file
   (format "(setq use-smtp \"%s\")\n"
           (setq use-smtp
                 (read-string
                  "What's your smtp email sending server name? "
                  (format "mail.%s" use-domain))))
   'nil pref-file))

;;address of your pop3 email server to GET mail:
(when (and (not (boundp 'use-pop3)) use-net)
  (append-to-file
   (format "(setq use-pop3 \"%s\")\n"
           (setq use-pop3
                 (read-string
                  "What's your pop3 email receiving server name? "
                  use-smtp)))
   'nil pref-file))

;;address of nntp usenet news reader
(when (and (not (boundp 'gnus-nntp-server)) use-net)
  (append-to-file
   (format "(setq gnus-nntp-server \"%s\")\n"
           (setq gnus-nntp-server
                 (read-string
                  "What's your nntp usenet news server name? "
                  (format "news.%s" use-domain))))
   'nil pref-file))

(defvar nntp-authinfo-user)		;shut up compiler
(when (and (not (boundp 'nntp-authinfo-user)) use-net)
  (append-to-file
   (format "(setq nntp-authinfo-user \"%s\")\n"
           (setq nntp-authinfo-user
                 (read-string
                  "What's your nntp usenet news login id? "
                  (format "%s" use-login-name))))
   'nil pref-file))

;;__________________________________________________________________________
;;;;;;		Printer
;;Common unix printer names:	"/dev/lp0"	"/dev/rp0"
;;typical dos printer names:	"prn"	"\\\\server\\mylaser"
;;'NET HELP' in ms windows will get you networking commands.
;;If you know your server name, 'NET VIEW SERVER' will list its printer(s).

;;(defconst printer-name "//print_server/hp")		;new emacs syntax
;;(defconst ps-printer-name "\\\\hpserver\\hp_ps")	;postscript

(defvar use-paper)
(when (not (boundp 'use-paper))
  (append-to-file
   (format "(defconst use-paper  \"%s\")\n"
	   (setq use-paper
		 (read-string
		  "Letter or a4 (metric) paper size? " "letter")))
   'nil pref-file))

;;Regular Printer. Sometimes predefined as 'nil.
(when (or (not (boundp 'printer-name)) (eq printer-name 'nil))
  (if (string-match "windows" (symbol-name system-type)) ;for ms windows
      (append-to-file
       (format "(setq printer-name  \"%s\")\n"
	       (setq printer-name
		     (read-string
		      "What's your printer name? (see control panel) "
		      "//no-server/no-printer-port-name"))) 'nil pref-file)
  ;;else for mac, linux, all others 
    (append-to-file
     (format "(setq printer-name  \"%s\")\n"
             (setq printer-name
                   (read-string "What's your printer name? "
                                "/dev/lp0"))) 'nil pref-file)))

;;Postscript Printer: use regular printer name for default
(defvar ps-printer-name)			;shut up compiler
(when (not (boundp 'ps-printer-name))
  (append-to-file
   (format "(setq ps-printer-name  \"%s\")\n"
	   (setq ps-printer-name
		 (read-string "What's your postscript printer name? "
			      printer-name))) 'nil pref-file))

;; (if
;;  (string-match "windows" (symbol-name system-type)) ;for ms windows
;;   (when (not (boundp 'ps-printer-name))
;;     (append-to-file
;;      (format "(setq ps-printer-name  \"%s\")\n"
;;              (setq ps-printer-name
;;                    (read-string "What's your postscript printer name? "
;;                                 "PRN"))) 'nil pref-file))

;; ;;else for mac, linux, all others 
;;   (when (not (boundp 'ps-printer-name))
;;     (append-to-file
;;      (format "(setq ps-printer-name  \"%s\")\n"
;;              (setq ps-printer-name
;;                    (read-string	"What's your postscript printer name? "
;; 				"/dev/lp0"))) 'nil pref-file)))

(autoload 'pp "pp" "Postscript Print" t) ;May ease some load bugs
(autoload 'pp-to-string "pp" "Postscript Printer settings" t)

(when (not (boundp 'lpr-command))	;note that undefined lpr-switches is ok
  (defconst lpr-list (split-string
                      (read-string
                       "What is your print command? "
                       (concat "lp -d " printer-name))))
  (setq lpr-command (car lpr-list))
  (append-to-file
   (format "(setq lpr-command \"%s\")\n" lpr-command) 'nil pref-file)

  (setq lpr-switches (cdr lpr-list))
  (defvar cache-pp)			;shut up compiler
  (cache-locate-library
   use-cache 'cache-pp "pp"
   "For postscript printer support, get pp.el from full tarball distribution where you got XEmacs")

  (when cache-pp
    ;;w32 xemacs 21.1.13 binary distrib has bad file "pp" (no .el) in lisp dir
    (condition-case err                 ;Handle any exceptions
	(require 'pp)
      (err (message err)(message "Fix 'pp.el' in your emacs lisp path")))
    (when (featurep 'pp)
      (append-to-file
       (format "(setq lpr-switches '%s)\n" (pp-to-string lpr-switches))
       'nil pref-file))))

;;__________________________________________________________________________
;;;;;;		Programmer
(defvar use-indent)			;shut up compiler
(when (not (boundp 'use-indent))	;number of spaces to indent
  (append-to-file
   (format "(setq use-indent %s)\n"
           (setq use-indent
                 (read-string "How many spaces should tabs indent? " "2")))
   'nil pref-file))

;;__________________________________________________________________________
;;;;;;		SQL Mode
;;Search PATH for a database, and configure for it, if possible

;;See Also w32.el, Where sql-oracle-program Is Set !
;;sql		<url:http://www.geocities.com/TimesSquare/6120/emacs.html>
;;sql-mode (good!) <url:http://www.stanford.edu/~riepel/SQL-mode.html>

(when (not (boundp 'use-sql-vendor))
  (cond
   ((which "sqlplus")                   ;all OS except w32
    (defconst use-sql-vendor "oracle")
    (append-to-file "(defconst use-sql-vendor \"oracle\")\n" 'nil pref-file))

   ((which "plus80")                    ;w32
    (defconst use-sql-vendor "oracle")
    (append-to-file "(defconst use-sql-vendor \"oracle\")\n" 'nil pref-file))

;;Search isql's PATH for words "microsoft" or "sybase"
   ((which "isql")
    (defvar sql-path)			;shut up compiler
    (setq sql-path (which-first "isql"))
    (cond
     ((string-match "sybase" sql-path)
      (defconst use-sql-vendor "sybase")
      (append-to-file "(defconst use-sql-vendor \"sybase\")\n" 'nil pref-file))
     ((string-match "microsoft" sql-path)
      (defconst use-sql-vendor "microsoft")
      (append-to-file "(defconst use-sql-vendor \"microsoft\")\n" 'nil
		      pref-file))
     ((or (string-match "borland" sql-path)
	  (string-match "interbase" sql-path))
      (defconst use-sql-vendor "interbase")
      (append-to-file "(defconst use-sql-vendor \"interbase\")\n" 'nil
		      pref-file))
     (t				;else prompt for vendor
      (append-to-file
       (format
	"(setq use-sql-vendor \"%s\")\n"
	(setq use-sql-vendor
	      (read-string
	       "What brand SQL do you use [microsoft, sybase, interbase]? "
	       "microsoft"))) 'nil pref-file))))

   ((which "mysql")
    (defconst use-sql-vendor "mysql")
    (append-to-file "(defconst use-sql-vendor \"mysql\")\n" 'nil pref-file))

   ((which "psql")
    (defconst use-sql-vendor "postgres")
    (append-to-file "(defconst use-sql-vendor \"postgres\")\n" 'nil pref-file))

   ((which "solsql")
    (defconst use-sql-vendor "solid")
    (append-to-file "(defconst use-sql-vendor \"solid\")\n" 'nil pref-file))
    
   ((which "dbaccess")
    (defconst use-sql-vendor "informix")
    (append-to-file "(defconst use-sql-vendor \"informix\")\n" 'nil pref-file))

   ((which "sql")
    (defconst use-sql-vendor "ingres")
    (append-to-file "(defconst use-sql-vendor \"ingres\")\n" 'nil pref-file))

   (t
    (defconst use-sql 'nil)
    (append-to-file
     "(defconst use-sql-vendor \"none\")    ;no database found in PATH\n" 'nil
     pref-file)
    (message
     "configure.el: No supported database program found in PATH. SQL support disabled"))))
;;Does anyone read these messages? If so, turn this on:
;;(message (concat (concat "Supported sql brand is " use-sql-vendor) "\n"))

;;Set and save the SQL server info, if it doesn't exist & SQL is enabled
(defvar sql-user)			;shut up compiler
(defvar sql-password)			;shut up compiler
(when (and (boundp 'sql-vendor) (not (boundp 'sql-user)))
  (append-to-file
   (format "(setq sql-user \"%s\")\n"
           (setq sql-user
                 (read-string "What is your SQL login name? "
			      (format "%s" use-login-name)))) 'nil pref-file)
  (when (not (string=
	      (setq sql-password
		    (read-string
		     "You may break security, and save your password: "
		     "[skip]")) "[skip]"))
    (append-to-file
     (concat (concat "(setq sql-password \"" sql-password) "\")\n")
     'nil pref-file)))

(when (and (boundp 'sql-vendor) (not (boundp 'sql-server)))
  (when (not (boundp 'use-domain))
    (when (getenv "MAILHOST")
      (setq use-domain
            (format "%s.%s" 
                    (car(cdr (split-string (getenv "MAILHOST") "[.]")))
                    (car(cdr(cdr (split-string (getenv "MAILHOST") "[.]")))))))

    (when (not (boundp 'use-domain))
      (setq use-domain
            (read-string "What's your network domain? " "localhost.com"))))

  (append-to-file
   (format "(setq sql-server \"%s\")\n"
           (setq sql-user
                 (read-string
                  "What is your database server host name? "
                  (format "sql.%s" use-domain)))) 'nil pref-file))

(when (and (boundp 'sql-vendor) (not (boundp 'sql-database)))
  (append-to-file
   (format "(setq sql-database \"%s\")\n"
           (setq sql-user
                 (read-string
                  "What is your database name on the server? " "master")))
   'nil pref-file))

;;__________________________________________________________________________
;;;;	Optional Packages
;;Many of these are not available on older emacs; see emacs.html to
;;download them.

;;__________________________________________________________________________
;;;;	Load Affiliated Files
(cache-locate-library
 use-cache 'cache-tinyload "tinyload"
 "To start EMacro faster, get tiny tools from <url: http://tinytools.sourceforge.net>")

(if cache-tinyload
    (progn
      (defvar tinyload-:load-list)	;shut up compiler
      (setq tinyload-:load-list
	    '("e-keys"			;Key assignments
	      "e-common"		;Remaining common Emacs & XEmacs code
;;e-c & e-perl are autoloaded by programmer.el
	      "e-java"
	      "e-xml"			;HTML, SGML, XML
	      ))

      (require 'e-programmer)		;General Programming

      (when use-net			;Internet support
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-net"))))

      (when (boundp 'sql-vendor)
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-sql"))))

      (cond
       ((string-match "GNU" (emacs-version))
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-macs")))
      (when (file-readable-p (concat emacro-top-dir "e-custom"))
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-custom")))))

       ((string-match "XEmacs" (emacs-version))
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-xemacs")))
	(when (file-readable-p (concat emacro-top-dir "e-xcustom"))
	  (setq tinyload-:load-list
		(append tinyload-:load-list '("e-xcustom"))))))

      (cond
       ((string-match "windows" (symbol-name system-type))
	(require 'e-w32))		;tinyload causes problems :(

      ((or (string-match "Macintosh" (emacs-version))
	   (string-match "macos" (emacs-version)))
       (setq tinyload-:load-list
	     (append tinyload-:load-list '("e-mac-os"))))

      (t (setq tinyload-:load-list (append tinyload-:load-list '("e-linux")))))

;;Your custom code goes in postload.el, which is loaded last:
      (when (not (featurep 'e-postload))
	(setq tinyload-:load-list
	      (append tinyload-:load-list '("e-postload")))
	(provide 'e-postload))

      (require 'tinyload))		;time delayed loading of list

;;Otherwise, no tinyload: go it the slow way:
  (progn
    (require 'e-keys)			;Key assignments
    (require 'e-common)			;Remaining common Emacs & XEmacs code
    (require 'e-programmer)		;General Programming
;;e-c & e-perl are autoloaded by programmer.el
    (require 'e-java)
    (when (boundp 'sql-vendor) (require 'e-sql))
    (require 'e-xml)			;HTML, SGML, XML
    (when use-net (require 'e-net))	;Internet routines

;;Figure which macros to load, by OS and XEmacs or Gnu Emacs.
    (cond
     ((string-match "GNU" (emacs-version)) ;FSF/Gnu Emacs, all platforms
      (require 'e-macs)
      (when (file-readable-p (concat emacro-top-dir "e-custom"))
	(load-library "e-custom")))

     ((string-match "XEmacs" (emacs-version)) ;XEmacs, all platforms
      (require 'e-xemacs)
      (when (file-readable-p (concat emacro-top-dir "e-xcustom"))
      (load-library "e-xcustom"))))

    (cond
     ((string-match "windows" (symbol-name system-type))
      (require 'e-w32))
     ((or (string-match "Macintosh" (emacs-version))
	  (string-match "macos" (emacs-version)))
      (require 'e-mac-os))
     (t (require 'e-linux)))		;Else only on unix

    (when (not (featurep 'e-postload))
      (load-library "e-postload")
      (provide 'e-postload))))		;User code (that's you!) 

;;__________________________________________________________________________
;;;;		Version Check
;;If you said you have a fast net connection, EMacro checks web for upgrades
;;EMacro checks known hardcoded X/Emacs versions. Results into splash screen

(defconst emacro-ver-uri "http://emacro.sf.net/emacro.ver")
(defconst emacro-is-old nil)

(when (and use-net (not use-modem))
;;See also links & other possible browsers
  (cond
   ((which "lynx")
    (when
	(ver-lessp
	 emacro-version
;;lynx -source works here the same as lynx -dump
	 (shell-command-to-string (concat "lynx -dump " emacro-ver-uri)))
      (setq emacro-is-old t)))
   ((which "links")
    (when
	(ver-lessp
	 emacro-version
	 (shell-command-to-string (concat "links -source " emacro-ver-uri)))
      (setq emacro-is-old t)))
   ))

(setq emacro-splash-text "\n")
(when (string-match "Gnu" (emacs-version))
  (when emacro-is-old
    (setq emacro-splash-text
	  (concat "\tAn EMacro UPGRADE is available!\n\n" emacro-splash-text)))

;;UPDATE EMACS VERSION HERE!
  (when (ver-lessp emacs-version "21.2.8")

    (setq emacro-splash-text
	  (concat "\tAn Emacs UPGRADE is available!\n\n" emacro-splash-text))))

(when (not (string= emacro-splash-text "\n"))
  (if (and (string= window-system "x") (string= system-type "gnu/linux"))
      (setq
       fancy-splash-text
       (append
	(list
	 (list
	  :face 'bold
	  emacro-splash-text))
	fancy-splash-text))
;;Else splash screens for systems not as X Window on Linux:
    (let ((buffer (generate-new-buffer "MESSAGE")))
      (add-text-properties 0 (length emacro-splash-text)
			   '(face bold) emacro-splash-text)
      (switch-to-buffer buffer)
      (insert "\n\n\n" emacro-splash-text)
      (set-buffer-modified-p nil)
      (sit-for 7)
      (kill-buffer buffer))))

(when (string-match "xemacs" (emacs-version))
;;Currently, 21.4.x are Gamma releases; the 21.x branch are stable.

;;UPDATE XEMACS VERSION HERE
  (when (ver-lessp emacs-version "21.1.14")

    (setq splash-frame-static-body
	  (append (list '(face bold "\tA XEmacs UPGRADE is available!\n"))
		  splash-frame-static-body)))
  (when emacro-is-old
    (setq splash-frame-static-body
	  (append (list '(face bold "\tAn EMacro UPGRADE is available!\n"))
		  splash-frame-static-body)))
  (xemacs-splash-buffer))		;Force splash screen to be displayed


;;__________________________________________________________________________
;;;;	Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-config)

;;; e-config.el ends here
