;;;; e-keys.el --- Keyboard routines for gnu and x emacs for every OS.
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <2002-06-15 15:24:33 bingalls>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;	TABLE OF CONTENTS
;; 	Legal
;; 	Commentary
;; 	Code
;;		Elisp Key Syntax
;;		Key Macro Hints
;;		Set CUA (Windows-Style) Key Bindings
;;		Tab
;;		Mouse Bindings
;;		Next & Previous Parens
;;		Assorted Global Key Bindings
;;		Text Selection
;;		Prevent Key Flubs
;;		TeraTerm Key Bindings
;;		-nw Terminal mode for Sparc keyboard
;;		Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;; Commentary:
;;Emacs editor startup file.  Keyboard routines for Gnu & X Emacs on every OS.
;;These are mostly routines for defining Control- and Meta- (alt or esc) key
;;combinations and Function keys.
;;Keybinding FAQ: <url:ftp://ftp.cs.uta.fi/pub/ssjaaa//emacs-keys.html>

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;; Keywords:		keyboard key binding map

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)

;;__________________________________________________________________________
;;;;	Elisp Key Syntax
;;both XEmacs and GNU/FSF Emacs
;;	(define-key global-map [(control right)] 'forward-word)
;;GNU/FSF Emacs only
;;	(define-key global-map [C-right] 'forward-word)
;;ver > 20, both
;;	(define-key global-map (kbd "C-<right>") 'forward-word)

;;NOTE: User keybindings should start with C-c
;;Any other keybinding risks breaking future Emacs compatibilty!

;;This should port to both Emacs and XEmacs:
;;(let ((keymap (make-sparse-keymap)))
;;  (define-key keymap "?\C-k" 'sl-kill-screen-line)
;;  ...
;;  (add-to-list 'minor-mode-map-alist keymap))

;;__________________________________________________________________________
;;;;	Key Macro Hints
;;View existing key commands	C-h b   or M-x help<ret> b
;;View recent keystrokes with	C-h l	or M-x view-lossage

;;__________________________________________________________________________
;;;;	Set CUA (Windows-Style) Key Bindings
;;See also CUA in e-xemacs.el, gnu.el & bugs in readme.txt

;;Moving cursor down at bottom scrolls only a single line, not half page
(setq scroll-step 1)
(setq scroll-conservatively 1)
;;Half-page scrolling is nice when viewing in the debugger

;;overrides beginning-of-line (home) to select entire active file
;;(global-set-key [(control a)] 'mark-whole-buffer)

;;C-s is reserved for search; use Alt-S to save:
(global-set-key [(meta s)]	'save-buffer)
(global-set-key [(meta o)]	'find-file)
(global-set-key [(control o)]	'find-file)

;;Try swapping backspace & delete, which affects help key. See e-xemacs.el
;;(load-library "keyswap")
;;(setq delete-key-deletes-forward t)	;another possible fix
;;(define-key function-key-map [delete] "\C-d") ;yet another way

(global-set-key (quote [home]) 'beginning-of-line)
(global-set-key (quote [end]) 'end-of-line)
(global-set-key [(control escape)] 'list-buffers)

;;Rebind C-z MS windows style, to undo.
;;If you uncomment this, you will lose the standard tiny-tools keybindings
;;Note that C-y (redo) is native Emacs Yank (paste)
(global-set-key [(control z)] 'undo)

;;Sun keyboards have Alt keys in addition to Meta, so we can do microsoft
;;style keybindings. Technically, this is not cua, for purists...
;;Note that some keybindings might be used as XEmacs menu accelerators
(when
    (string-match "sun" (emacs-version))
  (global-set-key [(alt c)] 'copy-primary-selection)
  (global-set-key [(alt s)] 'save-buffer)
  (global-set-key [(alt v)] 'yank-clipboard-selection)
  (global-set-key [(alt x)] 'kill-primary-selection)
  (global-set-key [(alt y)] 'redo)
  (global-set-key [(alt z)] 'undo)
  )

;;__________________________________________________________________________
;;;;	Tab
;;Tab moves to the next code indentation
;;Shift Tab moves to the next tab stop (usually a multiple of 8 spaces)
(global-set-key [(iso-left-tab)] 'tab-to-tab-stop)

;;Here's how to get tab to do a real tab, instead of indenting.
;;Best solution. Seperates tab key from C-i on ttys:
;;(define-key text-mode-map (kbd "TAB") 'self-insert-command)

;;Another solution
;;Change text-mode-hook to whatever mode, e.g. html-mode-hook.
;;(add-hook 'text-mode-hook
;;	  (lambda ()
;;	    (local-set-key [tab]
;;			   (lambda ()
;;			     (interactive)
;;			     (insert "\t")))))

;;You can also skip the hook and just do for everywhere
;;(global-set-key [tab] (lambda () (interactive) (insert "\t")))

;;Only works if some text is currently highlighted.
;;(global-set-key [(control meta y)] 'yank-and-indent)

;;__________________________________________________________________________
;;;;	Mouse Bindings
;;From: Andreas Kuhn <andreas.kuhn@brokat.com> This should allow left mouse
;; select/copy and middle mouse paste on W32, just like on Linux:
;;(defun mouse-track-drag-copy-to-kill (event count)
;;  "Copy the dragged region to the kill ring"
;;  (let ((region (default-mouse-track-return-dragged-selection event)))
;;    (when region
;;      (copy-region-as-kill (car region)
;;                           (cdr region)))
;;    nil))
;;(add-hook 'mouse-track-drag-up-hook 'mouse-track-drag-copy-to-kill)

;;On x emacs, you can select a box of text with M- or Shift-M- button 1
;;(Alt Left Mouse button dragging). Note that your window manager might
;;intercept the mouse and prevent emacs.

;;For gnu emacs, get rect-mouse.el from
;;<url:ftp://ls6-ftp.cs.uni-dortmund.de/pub/src/emacs/>
;;There's also rect-mark.el for gnu emacs; I had problems on old v20.2

(autoload 'delete-rectangle "rect" "cut a box of text with mouse" t)
(autoload 'delete-extract-rectangle "rect" "cut a box of text with mouse" t)
(autoload 'extract-rectangle "rect" "select a box of text with mouse" t)
(autoload 'kill-rectangle "rect" "cut a box of text with mouse" t)
(autoload 'yank-rectangle "rect" "paste a box of text with mouse" t)
(autoload 'insert-rectangle "rect" "paste a box of text with mouse" t)
(autoload 'open-rectangle "rect" "clear a box of text with mouse" t)
(autoload 'string-rectangle "rect" "paste a box of text with mouse" t)
(autoload 'clear-rectangle "rect" "clear a box of text with mouse" t)

;;XEmacs predefines meta left mouse button to visually select a box
;;Uncomment to do it yourself:
;;(global-set-key [(meta button1)] 'mouse-track-do-rectangle)   ;select

;;select box with mouse. M-button1 is used by sawfish, but works elsewhere
(define-key global-map [(control shift button1)] 'mouse-track-do-rectangle)
(define-key global-map [(control shift button2)] 'yank-rectangle) ;paste
(define-key global-map [(control shift button3)] 'kill-rectangle) ;cut

;;This is probably predefined for Gnu Emacs:
;;(define-key global-map [(meta shift button3)] 'imenu)	;major mode menu

;;Mouse Wheel. See also mwheel.el, part of XEmacs.
(define-key global-map [(button4)] 'scroll-up)
(define-key global-map [(button5)] 'scroll-down)

;;__________________________________________________________________________
;;;;	Next & Previous Parens
(global-set-key [(meta p)] (lambda ()
			     (interactive)
			     (re-search-backward "(\\|)" nil t)))
(global-set-key [(meta n)] (lambda ()
			     (interactive)
			     (re-search-forward "(\\|)" nil t)))

;;__________________________________________________________________________
;;;;	Assorted Global Key Bindings

;;If your backspace key invokes help, try setting your terminal, so it sends
;;DEL: 'stty erase ^?' might help. Variations of the following elisp may help:
;;(setq tty-erase-char ?\177)
;;(global-set-key "\M-?" 'help-command)
;;(global-set-key "\C-h" 'delete-backward-char)
;;(setq delete-key-deletes-forward t)

;;Control C prefix is good Emacs style, but C-M-g is more realistic
(global-set-key [(control c)(g)] 'goto-line) ;prompts for line to jump to
(global-set-key [(control meta g)] 'goto-line) ;prompts for line to jump to

;; Make control+pageup/down scroll the other buffer
(global-set-key [(control next)] 'scroll-other-window)
(global-set-key [(control prior)] 'scroll-other-window-down)

(global-set-key [(control c) (control o)] 'comment-region)

;;This binds word completions to Ctrl+Space
(autoload 'dabbrev-expand "dabbrev" "word completion" t)
;;(global-unset-key [(control \040)])	;doesn't seem to work
;;(global-set-key [(control \040)]	'dabbrev-completion)
;;(global-set-key [(control \040)]	'dabbrev-expand)

(global-set-key [(control /)]		'dabbrev-expand)

;;See the Shift-Tab binding to real tab binding above
;;(global-set-key [(shift iso-lefttab)]	'dabbrev-completion)

;;Meta arrow keys cycle windows. Consider next-window() without 'multiframe'
(global-set-key [(meta right)]	'next-multiframe-window)
(global-set-key [(meta left)]	'previous-multiframe-window)

;; Set up some f-key shortcuts
(global-set-key [f1]		'help)  ;like windows (cua?)
(global-set-key [(shift f1)]	'manual-entry)   ;like windows
(global-set-key [f2]		'save-buffer)
(global-set-key [f3]		'isearch-forward) ;like windows

(global-set-key [f4]		'kill-buffer)
(global-set-key [(control f4)]	'delete-window)
(global-set-key [(meta f4)]	'save-buffers-kill-emacs)

(global-set-key [f5]		'compile)
(global-set-key [f6]		'next-error)
(global-set-key [(shift f6)]	'first-error)
(global-set-key [f7]		'previous-error)

(global-set-key [f8]		'split-window-vertically)
(global-set-key [(shift f8)]	'delete-other-windows)
(global-set-key [f9]		'new-frame)
(global-set-key [(shift f9)]	'delete-frame)

;;F10 is menu for Gnu Emacs & windows
;;'quote' is optional syntax
(global-set-key (quote [f11])	'undo)
(global-set-key (quote [f12])	'other-window)
(global-set-key [(shift f12)]	'delete-window)

(global-set-key (quote [f14])	'undo)  ;undo key on sun keyboard
(global-set-key [(shift f14)]	'redo)

(global-set-key [(shift f16)]	'yank-line) ;copy key on sun keyboard

;;vi-style: hit % over paren to show matched paren
;;(global-set-key [(control %)] 'match-paren)
;;(global-set-key "%" 'goto-match-paren)
(global-set-key [(control %)] 'goto-match-paren)

;;(global-set-key [(meta g)] 'goto-line)	;override set-face
;;(global-set-key [(control /]) 'undo)
;;(global-set-key [(control _)] 'undo)

;;See e-functions.el for details
(global-set-key (read-kbd-macro "C-:") 'query-replace-last-two-kills)

;;__________________________________________________________________________
;;;;	Text Selection
(defconst query-replace-highlight t)	;highlight during query
(defconst search-highlight t)		;highlight incremental search

;;This may allow middle mouse pasting while doing a search. Needs testing.
;;(define-key isearch-mode-map (quote [mouse-2]) 'isearch-yank-kill)
;;(define-key isearch-mode-map (quote [down-mouse-2]) nil)

;;__________________________________________________________________________
;;;;	Prevent Key Flubs
;;(bad things that happen you slip onto the next key)
;;Make it easier to back out of hitting Esc key twice on older emacs
(put 'eval-expression 'disabled nil)

;;__________________________________________________________________________
;;;;	Teraterm Key Bindings
;;Uncomment this, for emacs inside w32 Teraterm ssh or telnet client from
;;<url: http://www.zip.com.au/~roca/download.html >. Recommended: _Setup
;;_Keyboard to transmit DEL by Backspace key & turn on Meta to enable Alt key.

;;This partially works, to detect we are inside a term:
;;(when (string= window-system nil)
;; (define-key function-key-map (kbd "ESC [ 6 ~") 'scroll-up)
;; (define-key function-key-map "\e[3~" 'scroll-down)
;; (define-key function-key-map "\e[5~" 'end-of-line)
;; (define-key function-key-map "\e[2~" 'beginning-of-line)
;; (define-key function-key-map "\e[1~" 'overwrite-mode))

;;__________________________________________________________________________
;;;;	-nw Terminal mode for Sparc keyboard
;;Here is a start at supporting emacs -nw for a sparc keyboard
;;Discover keys with 'C-h l'; discover keyboard type with (shell "kbd -t")
;;(define-key function-key-map (kbd "ESC [ C") [C-right])
;;(define-key function-key-map (kbd "ESC [ C") (kbd "C-<right>"))
;;(define-key function-key-map "\e[C" [(control right)])

;;__________________________________________________________________________
;;;;	Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-keys)
;;; e-keys.el ends here
