;;; xemacs.el --- Routines for XEmacs but not Gnu Emacs for every OS
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <02/05/22 17:34:32 bingalls>
;;$Id$

;;__________________________________________________________________________
;;TABLE OF CONTENTS
;;	Legal
;;	Commentary (Description)
;;	Code
;;	v21.2 Requirements
;;	Display & Menu
;;		Font Faces
;;		Mouse Click URLs
;;		Emacs Lisp Menu
;;		Menu Customization
;;		Function Menu
;;		Enable Xemacs Menu Items
;;	Keys
;;	Paren/Bracket/Brace Matching
;;	Address/Telephone Book
;;	Remote Editing & Ftp Interface
;;	Convert Dos ^M End Of Line
;;	Backups All In One Dir
;;	Auto Minibuffer (Status Line) Sizing
;;	Gnu Server
;;	Internet Relay Chat
;;	Telnet
;;	Tile Child Subwindows/Minibuffers Vertically
;;	Tiny Tools
;;	Toggle Menubar Function
;;	Toggle Modeline Function
;;	Toggle Toolbar Function
;;	Misc
;;	query-selected-replace

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;; Commentary:
;;Emacs editor startup file.  XEmacs routines for every OS

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://www.sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;; Keywords:		xemacs

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-xemacs nil "Settings from e-xemacs.el file." :group 'emacro)

;;__________________________________________________________________________
;;;; v21.2 Requirements
;;These are provided automatically in older versions
(require 'view-less)
(require 'cus-edit)                     ;Editing under M-x customize
(require 'help-macro)
(autoload 'info "info" "info" t)
(autoload 'hyper-apropos "hyper-apropos" "hyper-apropos" t)

;;__________________________________________________________________________
;;;;	Display & Menu
(defvar cache-big-menubar)		;shut up compiler
(cache-locate-library
 use-cache 'cache-big-menubar "big-menubar"
 "For complete XEmacs menu support, get big-menubar.el from full tarball distribution where you got XEmacs")
(when cache-big-menubar (require 'big-menubar))	;should be near beginning

;;shut up compiler
(autoload 'recent-files-initialize "recent-files" "recent file menu." t)
(defvar cache-recent-files)		;shut up compiler
(cache-locate-library
 use-cache 'cache-recent-files "recent-files"
 "For last opened files in menu support, get recent-files.el from full tarball distribution where you got XEmacs")

(when cache-recent-files
  (require 'dired)			;load before recent-files
  (require 'recent-files)		;open latest files from menu
  (setq recent-files-menu-path '("File")) ;make submenu of File, to save space

;;  (add-hook 'after-init-hook 'recent-files-initialize)

  (defcustom recent-files-add-menu-before "Open..."
    "Where 'Recent Files' appears in File menu."
    :group 'e-xemacs
    :type 'string)
  (recent-files-initialize))

(cache-locate-library
 use-cache 'cache-a2ps-print "a2ps-print"
 "For clean source code printouts, get a2ps-print.el from <url: ftp://ftp.enst.fr/pub/unix/a2ps/ >")

(when cache-a2ps-print
  (autoload 'a2ps-region "a2ps-print" "a2ps-print" t)
  (autoload 'a2ps-buffer "a2ps-print" "a2ps-print" t)
  (define-key function-key-map [f22] [print])
  (global-set-key [print] 'a2ps-buffer) ;f22 is Print Screen
  (global-set-key [(shift print)] 'a2ps-region)	;print selected text
  (add-menu-button '("File") ["a2ps Print Buffer" a2ps-buffer "--"]))

;;Remote printing example to put into postload.el:
;;(setq a2ps-command "ssh")
;;(setq a2ps-switches '("myHost" "a2ps" "-P" "myPSprinter" "--line-numbers=1"))

;;__________________________________________________________________________
;;;;;;		Font Faces
;;See x_custom.el, where these should be set
;;Sample font families: Fixed; Lucida Sans Typewriter; Interface User

;;(custom-set-faces
;;'(default ((t (:family "Screen" :size "14pt" :background "grey90"))) t))

;;__________________________________________________________________________
;;;;;;		Mouse Click URLs
;;See inet.el, where we define browse-url-at-mouse() to launch <url:...>
(cache-locate-library
 use-cache 'cache-overlay "overlay"
 "To Shift Middle-mouse click email addresses, get overlay.el from the full XEmacs sumo tarball")
(when cache-overlay
  (require 'overlay)
  (define-key global-map [(shift button2)] 'browse-url-at-mouse))

;;__________________________________________________________________________
;;;;;;		Emacs Lisp Menu
;;Enable compile from menu. Assumes emacs is in the path.
;;Note that starting another copy of emacs is not efficient, but at least
;;the menu does something relevant.
(add-hook
 'emacs-lisp-mode-hook
 (function
  (lambda ()
    (set (make-local-variable 'compile-command)
         (format "xemacs -batch -f batch-byte-compile %s" buffer-file-name))
    )))

;;__________________________________________________________________________
;;;;;;		Menu Customization
;;See ~lisp/edit-utils/big-menubar.el for more examples
(add-menu-button '("Options") ["Toggle Toolbar" toggle-toolbar])

;;Load any user defined toolbar
(if (and (featurep 'toolbar)
	 (fboundp 'console-on-window-system-p)
	 (console-on-window-system-p)
	 (file-exists-p "~/.xemacs/.toolbar"))
    (load-file (expand-file-name "~/.xemacs/.toolbar")))

;;Place version at help menu; launches html help
(add-menu-button '("Help") `[,(concat "EMacro v" emacro-version) emacro-help])

(defvar cache-redo)			;shut up compiler
(cache-locate-library
 use-cache 'cache-redo "redo"
 "For restoring undo support, get redo.el from sumo tarball distribution where you got XEmacs")

(when cache-redo
  (require 'redo)
  (add-menu-button '("Tools" "Utils")
                   ["Redo"	redo]))

(add-menu-button '("Tools" "Utils")
		 ["Refresh colors"	font-lock-fontify-buffer])
(add-menu-button '("Tools" "Utils")
		 ["Word completion"	dabbrev-expand])

;; Add 'dired' to the File menu
(add-menu-button '("File") ["Edit Directory" dired] "Insert File...")

;;The old way?
;;(add-submenu
;; '("Tools")		;change this line to 'nil' for top level menu
;; '("Utils" ["Redo"	redo	t] )
;; "----"					;menu seperator
;; )

;;__________________________________________________________________________
;;;;;;		Function Menu
;;menu to jump to any function definition
(defvar cache-func-menu)		;shut up compiler
(cache-locate-library
 use-cache 'cache-func-menu "func-menu"
 "For menu to jump to functions, get func-menu from sumo tarball distribution where you got XEmacs")

(when cache-func-menu
  (require 'func-menu)
  ;; (autoload 'fume-add-menubar-entry "func-menu" "function menu." t)

  (define-key global-map 'f8 'function-menu)
  (add-hook 'find-file-hooks 'fume-add-menubar-entry)

;;Define keys for these XEmacs only function menu functions
  (define-key global-map "\C-cl" 'fume-list-functions)
  (define-key global-map "\C-cg" 'fume-prompt-function-goto))

;;__________________________________________________________________________
;;;;;;		Enable Xemacs Menu Items
;;especially for win32 XEmacs. More mature XEmacs handle this already
(autoload 'xbm-button-create "xbm-button" "Required by file open dialog." t)
(autoload 'make-annotation "xpm-mode" "Required by file open dialog box." t)
(autoload 'xpm-mode "xpm-mode" "Required by file open dialog box." t)

(autoload 'shell "shell" "Start a command prompt in a buffer." t)
(autoload 'shell-mode "shell" "Start a command prompt in a buffer." t)
(autoload 'comint-xemacs "comint-mode" "Keep font in shell buffer." t)

;;This autoloads ediff-mode. Adds to tools menu: compare, merge, patch
(defvar cache-ediff-hook)		;shut up compiler
(cache-locate-library
 use-cache 'cache-ediff-hook "ediff-hook"
 "For comparing & merging files support, get ediff-hook from the full XEmacs distribution")

(when cache-ediff-hook
  (require 'ediff-hook)
  (add-hook 'ediff-cleanup-hook 'ediff-janitor)) ;add this to gnu.el?

;;(load-library "pretty-print")	;Not in core xemacs distribution!
(autoload 'pp-function "pp" nil t)
(autoload 'pp-variable "pp" nil t)
(autoload 'pp-plist     "pp" nil t)
(autoload 'macroexpand-sexp "pp" nil t)
(autoload 'macroexpand-all-sexp "pp" nil t)
(autoload 'prettyexpand-sexp "pp" nil t)
(autoload 'prettyexpand-all-sexp "pp" nil t)

(autoload 'grep "igrep" "Add igrep text search to tools menu." t)
(autoload 'igrep "igrep" "Add igrep text search to tools menu." t)
;;middle or double mouse button click of grep results jumps you into file

;;ToDo: Remove this deprecated tiny code
;; (defvar cache-tinyigrep)		;shut up compiler
;; (cache-locate-library
;;  use-cache 'cache-tinyigrep "tinyigrep"
;;  "For better text search support, get tinyigrep from <url: http://tiny-tools.sourceforge.net/ >")
;; (when cache-tinyigrep
;;   (use-prefix-key global-map "\C-z")		;need to fix this syntax
;;   (global-set-key "\C-zg" 'tinyigrep-menu))
;;   (autoload 'tinyigrep-menu "tinyigrep" "" t))

;; Put your customizations to separate file and add
;; (provide 'emacs-rc-tinyigrep) to the end of the resource file.
;;    (setq tinyigrep-:load-hook
;; 	    '(lambda ()
;; 	      (require 'emacs-rc-tinygrep "~/elisp/rc/emacs-rc-tinyigrep")))

;;__________________________________________________________________________
;;;;	Keys
;;(load-library "keyswap")
(setq delete-key-deletes-forward t)
(defcustom delete-key-deletes-forward t
  "Toggle ^? (del) key to delete next key."
  :group 'e-xemacs
  :type 'boolean)

;;Old XEmacs cua-mode locks up, when text is selected.
;;Uncomment the following code at your own risk 

;;(defvar cache-cua)		;shut up compiler
;;(cache-locate-library
;; use-cache 'cache-cua "cua-mode"
;; "For CUA key support, get old xemacs cua-mode.el version from <url:http://www.cua.dk/cua.html>")

;;(if cache-cua
;;    (progn
;;      (require 'cua-mode)
;;      (CUA-mode t))
;;  (progn

    (defvar cache-pc-select)		;shut up compiler
    (cache-locate-library
     use-cache 'cache-pc-select "pc-select"
     "For shift-arrow selection support, get pc-select.el from full tarball distribution where you got Emacs")

    (when cache-pc-select
      (require 'pc-select)                ;requires xemacs > v20.5
;;shut up compiler:
      (autoload 'pc-select-mode "pc-select" "Shift-Arrow highlight" t)
      (pc-select-mode t))                 ;Shift-Arrow highlighting

;;))					;end of cua code block

;;__________________________________________________________________________
;;;;	Paren/Bracket/Brace Matching
(autoload 'paren-set-mode "paren" "show matching parenthesis" t) ;quiet compile
(defvar cache-mic-paren)		;shut up compiler
(cache-locate-library
 use-cache 'cache-mic-paren "mic-paren"
 "For matching parenthesis, braces, etc. support, get paren.el and mic-paren.el from sumo tarball distribution where you got XEmacs")

(defvar cache-paren)			;shut up compiler
(cache-locate-library
 use-cache 'cache-paren "paren"
 "For matching parenthesis, braces, etc. support, get paren.el and mic-paren.el from sumo tarball distribution where you got XEmacs")

(cond (cache-mic-paren
       (require 'mic-paren)
       (defcustom paren-face 'bold "Matched paren hilight style."
	 :group 'e-xemacs
	 :type 'string)
       (defcustom paren-sexp-mode nil "Matched paren hilight style."
	 :group 'e-xemacs
	 :type 'boolean)
       (paren-activate))
      (cache-paren
       (require 'paren)
       ;;other modes on solaris XEmacs 21.1 leave annoying highlight ghosts.
       (paren-set-mode 'blink-paren)))

;;(paren-set-mode 'paren) ;highlight matching parenthesis. Also try 'sexp
;;(defconst paren-set-mode 'paren) ;hilite matched parens. Also try 'sexp

;;__________________________________________________________________________
;;;;	Address/Telephone Book Function
;;Gnu Emacs version provides (goto-address)
(global-set-key [(meta f3)] 'address-book)
(defun address-book ()
  "Opens .addressbook (with goto address support on gnu Emacs)."
  (interactive)  (find-file "~/emacs/addressbook"))

;;__________________________________________________________________________
;;;;	Email To Address At Cursor
;; Based on a defun by Kevin Rodgers <kevinr@ihs.com>
(autoload 'thing-at-point "thingatpt" "mouse enable object" t)
(autoload 'mail-abbrev-init-keys "mail-abbrevs" "mail abbrev." t)
(autoload 'mail "sendmail" "Mail." t)

(defvar email-regexp)                   ;shutup bytecompiler
(eval-when-compile
  (defun mail-to-address-at-point ()
    "*Edit a new mail message to the address at point."
    (interactive)
    (let ((url-at-point (substring (thing-at-point 'url) 7)))
      (if (string-match email-regexp url-at-point)
	  (mail nil url-at-point)
	(message "Bad email address")))))

;;__________________________________________________________________________
;;;;	Remote Editing & FTP Interface
;;Open with	C-x C-f		/username@host:/home/username/file.txt
(defvar cache-efs-auto)			;shut up compiler
(cache-locate-library
 use-cache 'cache-efs-auto "efs-auto"
 "For remote file editing support, get efs.el from full tarball distribution where you got XEmacs")

(when cache-efs-auto
  (require 'efs-auto)                   ;efs-auto autoloads efs.
  (defconst efs-default-user use-login-name) ;set in configure.el
  (defcustom backup-by-copying 't
    "Non-nil fixes EFS remote ~ backup permission problems."
    :group 'e-xemacs
    :type 'boolean)
  (defcustom efs-make-backup-files 'nil
    "Nil is an alternate fix for EFS *~ backup permissions."
    :group 'e-xemacs
    :type 'boolean)

;;Use ncftp for your ftp program (allows using a proxy)
  (when (which "ncftp")
    (defcustom efs-ftp-prompt-regexp "^.*ncftp.*>.*"
      "Regular expression which matches the FTP prompt."
      :group 'e-xemacs
      :type 'string)
    (defcustom efs-ftp-program-args nil "Nil works well for EFS and ncftp."
      :group 'e-xemacs
      :type 'boolean)
    (defcustom efs-ftp-program-name (which-first "ncftp")
      "FTP Program for remote editing."
      :group 'e-xemacs
      :type 'string))

;;Uncomment this, if you see "No such file or directory, nslookup"
;;(setq efs-generate-anonymous-password use-email-addr)
)

;;__________________________________________________________________________
;;;;	Convert Dos ^M End Of Line
;;(add-hook 'find-file-hooks
;;                  (lambda ()
;;                    (unless (eq
;;			(coding-system-eol-type buffer-file-coding-system) 'lf)
;;                      (set-buffer-file-coding-system
;;                       (subsidiary-coding-system
;;                        (coding-system-base buffer-file-coding-system) 'lf)
;;                       t))))

;;__________________________________________________________________________
;;;;	Backups All In One Dir
;;(require 'backup-dir)
;; (defconst bkup-backup-directory-info
;;       '(("/home/docs/.*" "/~/.backups/" ok-create full-path)
;; 	   (t                ".backups/"    full-path search-upward)))

;;__________________________________________________________________________
;;;;	Auto Minibuffer (Status Line) Sizing
;;resize-minibuffer-mode auto resizes the minibuffer to hold its contents
(autoload 'resize-minibuffer-mode "rsz-minibuf" nil t) ;shut up compiler

(condition-case err
    (progn
      (resize-minibuffer-mode)
      (defconst resize-minibuffer-window-exactly nil)
      )                                 ;else
  (error
   (progn
     (message "Cannot resize minibuffer %s." (cdr err))
     (message "For support to minimize the other buffer, get rsz-minibuf.el from full tarball distribution where you got XEmacs"))))

;;alternate, from
;; <url:ftp://ftp.splode.com/pub/users/friedman/emacs-lisp/rsz-mini.el>
;;(require 'rsz-minibuf)

;;__________________________________________________________________________
;;;;	Gnu Server
;;keeps emacs running for faster restarts.
(when (featurep 'which)
  (if (or (which "gnuserv") (which "gnuservw"))
      (progn
	(defconst gnuserv-done-function 'bury-buffer)
	(require 'gnuserv)
	(gnuserv-start))
    ;;else
    (message "Make a link from gnuserv in ~lib/xemacs[ver]/[architecture] to PATH")))

;;__________________________________________________________________________
;;;;	Internet Relay Chat
(defvar cache-zenirc)			;shut up compiler
(cache-locate-library
 use-cache 'cache-zenirc "zenirc"
 "For IRC chat support, get zenirc from <url:http://www.xemacs.org/Download/index.html>")

(when cache-zenirc
  (autoload 'zenirc "zenirc" "Internet relay chat." t)
  (defcustom zenirc-server-default "irc.debian.org" "Default chat server." 
    :group 'e-xemacs
    :type 'string))

;;__________________________________________________________________________
;;;;;;		Tile Child Subwindows/Minibuffers Vertically
;;(require 'cl)
(autoload 'cl "cl-add-hook" "cl lisp." t)

(defun display-buffers (buffers &optional horizontally)
  "Display BUFFERS in current frame tiled vertically.
If HORIZONTALLY is non-nil (prefix arg), tile horizontally."
  (interactive
   (list (loop for buff = (read-buffer "Buffer to display: " nil t)
           while (not (equal buff ""))
           collect buff)
         current-prefix-arg))
  (delete-other-windows)
  (when buffers
    (let* ((max (if horizontally
                    (window-width)
                  (window-height)))
           (min (if horizontally
                    window-min-width
                  window-min-height))
           (size (/ max (length buffers))))
      (or (<= min size)
          (error "Too many buffers to display simultaneously"))
      (switch-to-buffer (first buffers))
      (loop for buffer in (rest buffers)
        do
        (select-window (split-window nil size horizontally))
        (switch-to-buffer buffer)))))

;;__________________________________________________________________________
;;;;	Tiny Tools
;;ToDo: remove this deprecated tiny code
 (defvar cache-tinyurl)			;shut up compiler
 (cache-locate-library
  use-cache 'cache-tinyurl "tinyurl"
  "For support to click & launch urls, get tiny-tools from <URL:http://www.sourceforge.net/projects/tiny-tools/>")
;; (autoload 'tinyurl-mode "tinyurl" "tinyurl" t)

(cache-locate-library
 use-cache 'cache-advice "advice"
 "For browser support, get advice.el from full tarball distribution where you got XEmacs")
(autoload 'ad-add-advice "advice" "advice" t)
(defvar use-modem)			;shut up compiler

(when (and cache-advice (not (string= window-system "gtk")))
  (autoload 'tinyurl-mode "tinyurl" "click urls to lauch browser" t)
  (when cache-tinyurl
    (add-hook 'tinyurl-:load-hook  'tinyurl-install-to-packages)
    (autoload 'tinyurl-mode-1 "tinyurl" "click urls to lauch browser" t)
    (tinyurl-mode)
    (cond
     ((not use-modem)
      (defconst tinyurl-:plugged-function '(lambda () t))))))

;;__________________________________________________________________________
;;;;	Toggle Menubar Function
(defun toggle-menubar () "Hide/display text menu (usually at top)."
  (interactive)
  (set-specifier
   menubar-visible-p
   (not (specifier-instance menubar-visible-p))))

;;__________________________________________________________________________
;;;;	Toggle Modeline Function
(defun toggle-modeline () "Hide/display status bar (usually at bottom)."
  (interactive)
  (set-specifier
   has-modeline-p
   (not (specifier-instance has-modeline-p))))

;;__________________________________________________________________________
;;;;	Toggle Toolbar Function
(defun toggle-toolbar () "Hide/display icons (usually at top)."
  (interactive)
  (set-specifier
   default-toolbar-visible-p
   (not (specifier-instance default-toolbar-visible-p))))

;;__________________________________________________________________________
;;;;	Adjust Colors
;;This makes a nice syntax (font-lock) color on some monitors:
;;(defface font-lock-keyword-face
;;  '((((class color) (background dark)) (:foreground "cyan"))
;;    (((class color) (background light)) (:foreground "orange4"))
;;    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
;;    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
;;    (t (:bold t)))
;;  :group 'font-lock-faces)

;;(set-face-foreground font-lock-keyword-face orange4)

;;__________________________________________________________________________
;;;;	Misc
;;(defconst url-be-asynchronous t)	;Prevent waiting on W3 browser
(autoload 'ksh-mode "ksh-mode" "ksh mode." t)
;;(require 'ksh-mode)

(defvar cache-filladapt)		;shut up compiler
(cache-locate-library
 use-cache 'cache-filladapt "filladapt"
 "For comment wrapping, get filladapt.el from full tarball distribution where you got XEmacs")
(autoload 'filladapt-mode "filladapt" "filladapt" t)

(when cache-filladapt
  (require 'filladapt)                  ;Smart line wrapping of comments?
  (filladapt-mode t))

;;__________________________________________________________________________
;;;;	Telnet
;;(defconst ftelnet-program "/bin/telnet.exe")	;for w32
(autoload 'ftelnet "ftelnet" "telnet" t) ;requires xemacs > v20.5

;;__________________________________________________________________________
;;;;;;		query-selected-replace
;;From: scott.frazer@ericsson.com 
(defun query-selected-replace ()
  "query-replace, using the selected region, if active, as source to be replaced.
Otherwise, prompt the user, as usual"
  (interactive "*")
  (let (from-to-list from-string to-string)
    (if (region-active-p)
        (progn
          (setq from-string (buffer-substring (region-beginning) (region-end)))
          (zmacs-deactivate-region)
          (goto-char (region-beginning))
          (setq to-string (read-from-minibuffer
                           (format "Query replace %s with: " from-string)
                           nil nil nil 'query-replace-history))
          (progn
            (setq from-to-list (query-replace-read-args "Query replace" nil))
            (setq from-string (nth 0 from-to-list))
            (setq to-string   (nth 1 from-to-list))))
      (query-replace from-string to-string)))
  (global-set-key [(meta %)] 'query-selected-replace)) ;override original bind

;;__________________________________________________________________________
;;;;;;		yank-line
(defun yank-line () "copies current line. Should be merged with gnu emacs ver"
  (interactive)
  ;;This next line moves you to the beginning of line.
  (beginning-of-line)
  ;;Otherwise, this would just copy to end of line
  (call-interactively 'mark-end-of-line)
  (copy-primary-selection))

;;__________________________________________________________________________
;;;;	Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-xemacs)
;;; xemacs.el ends here
